@@ -148,7 +148,41 @@ Status SearchDBMRegex(
   assert(dbm != nullptr && matched != nullptr);
   std::unique_ptr<std::regex> regex;
   try {
-    regex = std::make_unique<std::regex>(pattern.begin(), pattern.end());
+    std::regex_constants::syntax_option_type options = std::regex_constants::optimize;
+    if (pattern.size() >= 2 && pattern[0] == '(' && pattern[1] == '?') {
+      bool ended = false;
+      size_t pos = 2;
+      while (!ended && pos < pattern.size()) {
+        switch (pattern[pos]) {
+          case 'a':
+            options |= std::regex_constants::awk;
+            break;
+          case 'b':
+            options |= std::regex_constants::basic;
+            break;
+          case 'e':
+            options |= std::regex_constants::extended;
+            break;
+          case 'i':
+            options |= std::regex_constants::icase;
+            break;
+          case 'l':
+            options |= std::regex_constants::egrep;
+            break;
+          case ')':
+            ended = true;
+            break;
+          default:
+            throw std::regex_error(std::regex_constants::error_complexity);
+        }
+        pos++;
+      }
+      if (!ended) {
+        throw std::regex_error(std::regex_constants::error_complexity);
+      }
+      pattern = pattern.substr(pos);
+    }
+    regex = std::make_unique<std::regex>(pattern.begin(), pattern.end(), options);
   } catch (const std::regex_error& err) {
     return Status(Status::INVALID_ARGUMENT_ERROR, StrCat("invalid regex: ", err.what()));
   }
@@ -267,6 +301,12 @@ Status SearchDBMModal(
     status = SearchDBMEditDistance(dbm, pattern, matched, capacity);
   } else if (mode == "editbin") {
     status = SearchDBMEditDistanceBinary(dbm, pattern, matched, capacity);
+  } else if (mode == "containcase") {
+    status = SearchDBM(dbm, pattern, matched, capacity, StrCaseContains);
+  } else if (mode == "containword") {
+    status = SearchDBM(dbm, pattern, matched, capacity, StrWordContains);
+  } else if (mode == "containcaseword") {
+    status = SearchDBM(dbm, pattern, matched, capacity, StrCaseWordContains);
   } else if (mode == "upper") {
     status = SearchDBMOrder(dbm, pattern, true, false, matched, capacity);
   } else if (mode == "upperinc") {
@@ -504,7 +544,41 @@ Status SearchTextFileRegex(
   }
   std::unique_ptr<std::regex> regex;
   try {
-    regex = std::make_unique<std::regex>(pattern.begin(), pattern.end());
+    std::regex_constants::syntax_option_type options = std::regex_constants::optimize;
+    if (pattern.size() >= 2 && pattern[0] == '(' && pattern[1] == '?') {
+      bool ended = false;
+      size_t pos = 2;
+      while (!ended && pos < pattern.size()) {
+        switch (pattern[pos]) {
+          case 'a':
+            options |= std::regex_constants::awk;
+            break;
+          case 'b':
+            options |= std::regex_constants::basic;
+            break;
+          case 'e':
+            options |= std::regex_constants::extended;
+            break;
+          case 'i':
+            options |= std::regex_constants::icase;
+            break;
+          case 'l':
+            options |= std::regex_constants::egrep;
+            break;
+          case ')':
+            ended = true;
+            break;
+          default:
+            throw std::regex_error(std::regex_constants::error_complexity);
+        }
+        pos++;
+      }
+      if (!ended) {
+        throw std::regex_error(std::regex_constants::error_complexity);
+      }
+      pattern = pattern.substr(pos);
+    }
+    regex = std::make_unique<std::regex>(pattern.begin(), pattern.end(), options);
   } catch (const std::regex_error& err) {
     return Status(Status::INVALID_ARGUMENT_ERROR, StrCat("invalid regex: ", err.what()));
   }
@@ -604,6 +678,12 @@ Status SearchTextFileModal(
     status = SearchTextFileEditDistance(file, pattern, matched, capacity);
   } else if (mode == "editbin") {
     status = SearchTextFileEditDistanceBinary(file, pattern, matched, capacity);
+  } else if (mode == "containcase") {
+    status = SearchTextFile(file, pattern, matched, capacity, StrCaseContains);
+  } else if (mode == "containword") {
+    status = SearchTextFile(file, pattern, matched, capacity, StrWordContains);
+  } else if (mode == "containcaseword") {
+    status = SearchTextFile(file, pattern, matched, capacity, StrCaseWordContains);
   } else {
     status = Status(Status::INVALID_ARGUMENT_ERROR, "unknown mode");
   }
